﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Controls.Editors;
using System.Collections.Specialized;
using System.Windows.Interactivity;
using System.Windows;
using System.Collections;

namespace XamComboEditorBehavior.Behaviors
{
    public class XamComboEditorSelectedItemsBehavior : Behavior<XamMultiColumnComboEditor>
    {
        private XamMultiColumnComboEditor Combobox
        {
            get { return AssociatedObject as XamMultiColumnComboEditor; }
        }

        public INotifyCollectionChanged SelectedItems
        {
            get { return (INotifyCollectionChanged)GetValue(SelectedItemsProperty); }
            set { SetValue(SelectedItemsProperty, value); }
        }
      
        public static readonly DependencyProperty SelectedItemsProperty = DependencyProperty.Register(
            "SelectedItems",
            typeof(INotifyCollectionChanged),
            typeof(XamComboEditorSelectedItemsBehavior),
            new PropertyMetadata(OnSelectedItemsPropertyChanged));

        private static void OnSelectedItemsPropertyChanged(DependencyObject target, DependencyPropertyChangedEventArgs args)
        {
            var collection = args.NewValue as INotifyCollectionChanged;
            if (collection != null)
            {
                //You should set the SelectedItems of the ComboBox when the SelectedItems property is changed
                XamComboEditorSelectedItemsBehavior behavior = target as XamComboEditorSelectedItemsBehavior;

                behavior.UnsubscribeFromEvents();
                XamComboEditorSelectedItemsBehavior.Transfer(collection as IList, behavior.Combobox.SelectedItems);
                behavior.SubscribeToEvents();
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            Combobox.SelectedItems.CollectionChanged+=SelectedItems_CollectionChanged;
        }

        void ContextSelectedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();
            Transfer(SelectedItems as IList, Combobox.SelectedItems);
            SubscribeToEvents();
        }

        void SelectedItems_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            UnsubscribeFromEvents();
            Transfer(Combobox.SelectedItems, SelectedItems as IList);
            SubscribeToEvents();
        }

        private void SubscribeToEvents()
        {
            Combobox.SelectedItems.CollectionChanged += SelectedItems_CollectionChanged;

            if (SelectedItems != null)
            {
                SelectedItems.CollectionChanged += ContextSelectedItems_CollectionChanged;
            }
        }

        private void UnsubscribeFromEvents()
        {
            Combobox.SelectedItems.CollectionChanged -= SelectedItems_CollectionChanged;

            if (SelectedItems != null)
            {
                SelectedItems.CollectionChanged -= ContextSelectedItems_CollectionChanged;
            }
        }

        public static void Transfer(IList source, IList target)
        {
            if (source == null || target == null) { return; }

            target.Clear();

            foreach (var o in source)
            {
                target.Add(o);
            }
        }
    }
}

